import { FastifyPluginAsync } from 'fastify';
import * as fsp from 'node:fs/promises';

import { Config, config } from '../config.js';
import { InvalidAuthorization, NotFoundError } from '../errors.js';
import { getFilesFor, getPathFor } from '../store.js';

export default (async function (fastify) {
    fastify.get('/get/:name', async (req, rep) => {
        const key = req.headers.authorization?.replace('Bearer ', '') ?? 'none';

        const client = Config.getClientByKey(config, key);
        if (!client) {
            throw InvalidAuthorization();
        }

        const files = (await getFilesFor(client.name)).map((x) =>
            x.replace(client.name + '-', ''),
        );

        const params = req.params as { name: string };

        if (!files.find((x) => x === params.name)) {
            throw NotFoundError();
        }

        rep.type('application/octet-stream');
        return await fsp.readFile(getPathFor(params.name, client.name));
    });
} as FastifyPluginAsync);
