import { type FastifyPluginAsync } from 'fastify';
import fs from 'fs';
import yaml from 'yaml';

const page = `<!DOCTYPE html>
<html>
    <head>
        <title>Backup server API reference</title>
        <link rel="stylesheet" href="https://unpkg.com/swagger-ui-dist@5.11.0/swagger-ui.css" />
    </head>
    <body>
        <div id="swagger-ui"></div>
        <script src="https://unpkg.com/swagger-ui-dist@5.11.0/swagger-ui-bundle.js" crossorigin></script>
        <script>window.onload=()=>window.ui=SwaggerUIBundle({url:'/openapi.json',dom_id:'#swagger-ui'})</script>
    </body>
</html>
`
    .replaceAll('\n', '')
    .replaceAll(/ +/gm, ' ')
    .replaceAll(' <', '<');

const openapiYml = yaml.parse(fs.readFileSync('openapi.yml').toString('utf-8'));

export default (async function (fastify) {
    fastify.get('/openapi', async (req, rep) => {
        rep.type('text/html; charset=utf-8');
        return page;
    });

    fastify.get('/openapi.json', async (req, rep) => {
        rep.type('application/json; charset=utf-8');
        return openapiYml;
    });
} as FastifyPluginAsync);
